package es.verdnatura.presentation.view.feature.login.model

class LoginSalixVO(
    val user: String = "",
    val password: String = "",
    val token: String = "",
    val created: String = "",
    val ttl: Long = 0,
    var isError: Boolean = false,
    var errorMessage: String = ""
)

class DataUserSalix(
    val id: Int? = null,
    val name: String = "",
    val nickname: String = "",
    var isError: Boolean = false,
    var errorMessage: String = ""
)

class RenewToken(
    val id: String,
    val ttl: Long = 0,
    var isError: Boolean = false,
    var errorMessage: String = ""
)

class SalixMessageVO(
    val workerId: Int = 0,
    val message: String = "",
    val token: String = "",
    var isError: Boolean = false,
    var errorMessage: String = ""
)

class SalixGrupo(
    val to: String = "",
    val message: String = ""

)
//Tarea 5870
class OperatorSalix(
    val sector: Sector? = null,
    val train: Train? = null,
    val printer: Printer? = null,
    val numberOfWagons: Int = 0,
    var itemPackingTypeFk: String? = null,
    val isError: Boolean = false,
    var errorMessage: String = ""
)
data class Sector(val id: Int, val description: String, val warehouseFk: Int)
data class Printer(val id: Int, val name: String)
data class Train(val id: Int, val name: String)


class accessConfigSalix(
    val id: Number = 0,
    val renewPeriod: Long = 0,
    val renewInterval: Long = 0,
    val isError: Boolean = false,
    var errorMessage: String = ""
)

class OperatorList(
    var list: List<OperatorSalix> = listOf()
)

class accessConfigSalixList(
    var list: List<accessConfigSalix> = listOf()
)

class Operator(
    val sectorFk: Int?,
    val labelerFk: Int?

)

class versionApp(
    val appname: String = "",
    val version: String = "",
    var IsVersionCritical: Int = 0,
    var url: String? = null,
    var isError: Boolean = false,
    var errorMessage: String = ""
)

class LoginDevice(
    val vMessage: String? = "",
    val vIsAuthorized: String = "",
    var isError: Boolean = false,
    var errorMessage: String = ""

)

class workerId(
    val Id: Int? = null,
    var isError: Boolean = false,
    var errorMessage: String = ""
)